<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Handler.php,v 1.3 2007-03-30 11:40:07 mike Exp $
 */

/**#@+
 * Check constants
 */
define('GADGET_HANDLER_CHECK_NONE',     0);
define('GADGET_HANDLER_CHECK_ISSET',    1);
define('GADGET_HANDLER_CHECK_TRUE',     2);
/**#@-*/

/**
 * Gadget_Handler
 *
 * KISS handler.
 * 
 * @package     Gadget
 * @since       1.0
 */
class Gadget_Handler
{
    /**
     * Handler Object
     * 
     * @var object
     */
    var $handlerObject;
    
    /**
     * Handler Class
     * 
     * @var string
     */
    var $handlerClass;
    
    /**
     * Registered Actions
     * 
     * @var array
     */
    var $actions = array();
    
    /**
     * Exclusive -- whether just one (first) or many actions can be handled
     * 
     * @var bool
     */
    var $exclusive = false;
    
    /**
     * Set Handler Object
     * 
     * @access  public
     * @return  void
     * @param   object  $handlerObject
     */
    function setObject(&$handlerObject)
    {
        $this->handlerObject = &$handlerObject;
    }
    
    /**
     * Set Handler Class
     * 
     * @access  public
     * @return  void
     * @param   string  $handlerClass
     */
    function setClass($handlerClass)
    {
        $this->handlerClass = $handlerClass;
    }
    
    /**
     * Handle
     * 
     * @access  public
     * @return  bool
     * @param   mixed   $action
     * @param   array   $params
     * @param   bool    $strict
     */
    function handle($action, $params, $check = GADGET_HANDLER_CHECK_ISSET)
    {
        if ($check & GADGET_HANDLER_CHECK_TRUE) {
            foreach ((array) $params as $condition) {
                if (!$condition) {
                    return false;
                }
            }
        } elseif ($check & GADGET_HANDLER_CHECK_ISSET) {
            foreach ((array) $params as $condition) {
                if (!isset($condition)) {
                    return false;
                }
            }
        }
        
        if (is_string($action)) {
            if (isset($this->handlerObject)) {
                $this->actions[] = array(
                    'action' => array(&$this->handlerObject, $action), 
                    'params' => $params,
                );
                return true;
            }
            if (isset($this->handlerClass)) {
                $this->actions[] = array(
                    'action' => array($this->handlerClass, $action),
                    'params' => $params,
                    'doload' => true
                );
                return true;
            }
        }
        
        $this->actions[] = compact('action', 'params');
        return true;
    }
    
    /**
     * Start
     * 
     * @access  public
     * @return  array
     */
    function start()
    {
        for ($i = 0, $c = count($this->actions); $i < $c; ++$i) {
            if (!empty($this->actions[$i]['doload'])) {
                $this->loadClass($this->actions[$i]['action'][0]);
            }
            if (is_callable($this->actions[$i]['action'])) {
                Gadget::log('Handler', $this->actions[$i]);
                call_user_func_array(
                    $this->actions[$i]['action'], 
                    $this->actions[$i]['params']
                );
                if ($this->exclusive) {
                    return $this->actions[$i];
                }
                unset($this->actions[$i]);
            }
        }
        return $this->actions;
    }
    
    /**
     * Load Class
     * 
     * @access  protected
     * @return  void
     * @param   string  $class
     */
    function loadClass($class)
    {
        @include_once strtr('_', '/', $class) .'.php';
    }
    
}

?>
