<?php
/**
 * Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Object.php,v 1.2 2005-04-15 13:07:46 mike Exp $
 */

/**
 * Gadget_Object
 * 
 * @package Gadget
 * @since   1.0
 */
class Gadget_Object
{
    /**
     * Constructor
     * 
     * @access  public
     * @param   array   $options, ...
     */
    function Gadget_Object($opts = array())
    {
        if (func_num_args() > 1) {
            $args = func_get_args();
            $opts = call_user_func_array('array_merge_recursive', $args);
        }
        $this->onBeforeConstruct($opts);
        $this->setOptions($opts);
        $this->onAfterConstruct($opts);
    }
    
    /**
     * On Before Construct
     * 
     * @access  protected
     * @return  void
     */
    function onBeforeConstruct()
    {
    }
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
    }
    
    /**
     * Set Options
     * 
     * @access  public
     * @return  void
     * @param   array   $options
     */
    function setOptions($options)
    {
        foreach ((array) $options as $property => $value) {
            if (    isset($this->$property) && 
                    is_array($value) &&
                    is_array($this->$property)) {
                $this->$property = array_merge($this->$property, $value);
            } else {
                $this->$property = $value;
            }
        }
    }
}

?>
