<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Configuration.php,v 1.3 2007-03-30 11:40:07 mike Exp $
 */

/**
 * Requires Gadget_Provider 
 */
require_once 'Gadget/Provider.php';

/**
 * Gadget_Provider_Configuration
 *
 * Gadget's default configuration provider
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @since       1.0
 */
class Gadget_Provider_Configuration extends Gadget_Provider
{
    var $provider = PROVIDER_CONFIGURATION;
    var $configuration;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     * @param   mixed
     */
    function init($conf = null)
    {
        if (!isset($conf)) {
            $conf = GADGET_CONFIG;
        }
        
        if (is_file($conf)) {
            $ini = parse_ini_file($conf, true);

            if (!isset($ini['Gadget'])) {
                die('No Gadget configuration defined!');
            }

            foreach ($ini as $section => $options) {
                foreach ($options as $k => $v) {
                    if (preg_match('/(?!\\\\)\$(\w+)/', $v, $m)) {
                        $v = preg_replace('/(?!\\\\)\$(\w+)/e', 
                            '$this->configuration["Gadget"]["\1"]', $v);
                    }
                    $this->configuration[$section][$k] = $v;
                }
            }
        } elseif (is_array($conf) || is_object($conf)) {
            if (!isset($conf['Gadget'])) {
                die('No Gadget configuration defined');
            }
            $this->configuration = $conf;
        }
    }
    
    /**
     * Get
     * 
     * @access  public
     * @return  mixed
     * @param   string
     * @param   string
     */
    function get($conf = null, $part = null)
    {
        if (isset($conf)) {
            if (isset($part)) {
                return isset($this->configuration[$conf][$part]) ? 
                    $this->configuration[$conf][$part] : null;
            }
            return isset($this->configuration[$conf]) ? 
                $this->configuration[$conf] : null;
        }
        
        return $this->configuration;

    }
    
    /**
     * Set
     * 
     * @note    Attention! Changes made through this ConfigurationProvider's set
     *          method are *NOT* permanent, i.e. they won't be saved!
     * 
     * @access  public
     * @return  void
     * @param   string
     * @param   array
     */
    function set($conf, $parts)
    {
        foreach ((array) $parts as $key => $value) {
            if (is_array($value) && @is_array($this->configuration[$conf][$key])) {
                $this->configuration[$conf][$key] = array_merge(
                    $this->configuration[$conf][$key], $value
                );
            } else {
                $this->configuration[$conf][$key] = $value;
            }
        }
    }
}
?>
