<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.6 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Currentuser.php,v 1.6 2007-03-30 11:40:07 mike Exp $
 */

/**
 * Requires Gadget_Provider 
 */
require_once 'Gadget/Provider.php';

/**
 * Gadget_Provider_Currentuser
 *
 * Gadget's default current user provider.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @since       1.0
 */
class Gadget_Provider_Currentuser extends Gadget_Provider
{
    var $provider = PROVIDER_CURRENTUSER;
    var $currentuser;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init($force = false)
    {
        if (isset($_SESSION['currentuser'])) {
            if ($_SESSION['currentuser']) {
                $this->currentuser = Gadget::DAO('users');
                if ($force || !isset($_SESSION['currentuserdata'])) {
                    $this->currentuser->selectAs();
                    $this->currentuser->joinAdd(Gadget::DAO('preferences'), 'LEFT');
                    $this->currentuser->selectAs(array('data'), 'pref%s', 'preferences');
                    $this->currentuser->get($_SESSION['currentuser']);
                    $_SESSION['currentuserdata'] = $this->currentuser->toArray();
                } else {
                    $this->currentuser->id = $_SESSION['currentuser'];
                    $this->currentuser->setFrom($_SESSION['currentuserdata']);
                    $this->currentuser->prefdata = $_SESSION['currentuserdata']['prefdata'];
                }
            }
        } else {
            $_SESSION['currentuser'] = 0;
        }
    }
    
    /**
     * Get
     * 
     * @access  public
     * @return  mixed
     * @param   mixed
     */
    function get($get = false)
    {
        if (!$this->currentuser) {
            $this->init();
        }
        if ($get) {
            if ($this->currentuser && is_string($get)) {
                return $this->currentuser->$get;
            }
            return $this->currentuser;
        }
        return $_SESSION['currentuser'];
    }
    
    /**
     * Set
     * 
     * @access  public
     * @return  void
     * @param   int
     */
    function set($id)
    {
        Gadget::log('DEBUG', 'CURRENT USER SET TO: '. $id);
        $_SESSION['currentuser'] = (int) $id;
        $this->init(true);
    }
    
}

?>
