<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Redirect.php,v 1.2 2007-03-30 11:40:07 mike Exp $
 */

/**
 * Requires Gadget
 */
require_once 'Gadget.php';

/**
 * Gadget_Redirect
 * 
 * @package Gadget
 * @since   1.2
 */
class Gadget_Redirect
{
    /**
     * Gadget_Redirect
     * 
     * @access  public
     * @param   string  $uri URI to redirect to.
     * @param   array   $params Parameters to append to the query string.
     * @param   bool    $session Whether to append session name and id.
     */
    function Gadget_Redirect($uri, $params = array(), $session = false)
    {
        Gadget::log('REDIRECT', $uri);
        if ($params and $charset = Gadget::getPref('charset')) {
            require_once 'I18Nv2.php';
            I18Nv2::recursiveIconv($params, GADGET_CHARSET, $charset);
        }
        require_once 'HTTP/Header.php';
        HTTP_Header::redirect($uri, $params, $session);
    }
    
    /**
     * Redirect to Login
     * 
     * Redirects to the configured login page.
     * 
     * @see Gadget_Provider_Configuration
     * 
     * @static
     * @access  public
     * @return  void
     */
    function login()
    {
        new Gadget_Redirect(Gadget::getConf('Gadget', 'login'), array('uri' => 
            isset($_GET['uri']) ? $_GET['uri'] : $_SERVER['REQUEST_URI']));
    }
    
    /**
     * Redirect Internal
     * 
     * Issues an internal redirect.  That means that the session will be 
     * preserved and the redirect points directly to the specified URL.
     * 
     * @see Gadget_Redirect::Gadget_Redirect()
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $uri URI to redirect to.
     * @param   array   $params Parameters to append to the query string.
     */
    function internal($uri = null, $params = array())
    {
        require_once 'Gadget/Cookie.php';
        isset($uri) or $uri = Gadget::location();
        new Gadget_Redirect($uri, $params, !Gadget_Cookie::get(
            session_name(), false));
    }
    
    /**
     * Redirect External
     * 
     * Issues an external redirect.  That means that the session will be 
     * removed and the redirect points to a configured redirection page.
     * This ensures that eventually existing session information will be
     * stripped from the url and will *not* show up in the referrer log
     * of the foreign server.
     * 
     * @see Gadget_Redirect::Gadget_Redirect()
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $uri URI to redirect to.
     */
    function external($uri)
    {
        new Gadget_Redirect(Gadget::getConf('Gadget', 'redirect'), 
            array('uri' => $uri), false);
    }
    
    /**
     * Redirect Back
     * 
     * Attempts to redirect back to previous page.
     * 
     * @access  public
     * @return  void
     */
    function back($upperLocation = false)
    {
        if (!$upperLocation) {
            $domain = Gadget::getConf('Gadget', 'wwwdomain');
            if (isset($_SERVER['HTTP_REFERER'])) {
                if (preg_match('/^https?:\/\/'. $domain .'/i', 
                        $_SERVER['HTTP_REFERER'])) {
                    return Gadget_Redirect::internal($_SERVER['HTTP_REFERER']);
                } else {
                    new Gadget_Redirect($_SERVER['HTTP_REFERER']);
                }
            }
        } else {
            $location = Gadget::location();
            return Gadget_Redirect::internal(
                substr($location, 0, strrpos($location, '/')));
        }
    }
}
?>