<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Errorlog.php,v 1.3 2007-03-30 11:40:08 mike Exp $
 */

/**
 * Requires Gadget_Service
 */
require_once 'Gadget/Service.php';

/**
 * Gadget_Service_Errorlog
 *
 * Gadget's default error log service.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Errorlog extends Gadget_Service
{
    var $service = SERVICE_ERRORLOG;
    var $log;
    
    /**
     * Init
     * 
     * @access  public
     * @return  void
     */
    function init()
    {
        require_once 'Log.php';
        $this->log = Log::factory(
            Gadget::getConf('Error', 'log.driver'),
            Gadget::getConf('Error', 'log.destination'),
            'GADGET'
        );
    }
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     * @param   object
     */
    function execute($error)
    {
        if (!$this->log) {
            return false;
        }
        return $this->log->err(
            sprintf(
                '[%s@%s] %s (%s) %s', 
                Gadget::currentUser('alias'), Gadget::location(),
                str_replace("\n", ' ', $error->getMessage()), 
                $error->getCode(),
                str_replace("\n", ' ', $error->getUserInfo())
            )
        );
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure()
    {
        Gadget::onError(Gadget::onError() &~ E_LOG);
        PEAR::raiseError('Gadget::onError(E_LOG) has been disabled!');
        return false;
    }
    
}

?>
