<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.8 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Errornotify.php,v 1.8 2009-11-09 19:41:14 mike Exp $
 */

/**
 * Requires Gadget_Service
 */
require_once 'Gadget/Service.php';

/**
 * Gadget_Service_Errornotify
 *
 * Gadget's default error notify service.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Errornotify extends Gadget_Service
{
    var $service = SERVICE_ERRORNOTIFY;
    var $mail;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init()
    {
        require_once 'Mail.php';
        $opts = Gadget::getConf('Mail');
        $this->mail = &Mail::factory($opts['driver'], $opts);
    }
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     * @param   object
     */
    function execute($error)
    {
        if (PEAR::isError($this->mail)) {
            return false;
        }
        
        $opts = Gadget::getConf('Mail');
        
        require_once 'HTTP.php';
        require_once 'Mail/mime.php';
        $mime = new Mail_mime();
        $mime->setTXTBody($error->getMessage()."\n\n".$error->getUserInfo());
        $mime->addAttachment(print_r($error, true), 'text/plain', 'error.txt', false);
        $mime->setFrom($opts['admin']);
        $mime->setSubject('PEAR Error at '. HTTP::absoluteUri());
        
        $body = $mime->get();
        $hdrs = $mime->headers(array('Date' => gmdate('r'), 'To' => $opts['admin']));
        
        // send mail
        return !PEAR::isError($this->mail->send($opts['admin'], $hdrs, $body));
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure()
    {
        // disable E_NOTIFY if we failed
        Gadget::onError(Gadget::onError() &~ E_NOTIFY);
        PEAR::raiseError('Gadget::onError(E_NOTIFY) has been disabled!');
        return false;
    }
    
}

?>
