<?php

require_once 'Gadget/Service.php';

/**
 * Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.13 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Mailaccount.php,v 1.13 2007-03-31 06:46:06 mike Exp $
 */

/** 
 * Gadget_Service_Mailaccount
 *
 * Gadget Service for Mail Account Operations.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Mailaccount extends Gadget_Service
{
    /**
     * Service
     * 
     * @var string
     */
    var $service = SERVICE_MAILACCOUNT;
    
    /**
     * Occured Fault Message
     * 
     * @var string
     */
    var $fault;
    
    /**
     * XML_RPC Client
     * 
     * @var object
     */
    var $client;
    
    /**
     * XMLRPC Method
     * 
     * @var string
     */
    var $method;
    
    /**
     * Mail Account
     * 
     * @var string
     */
    var $account;
    
    /**
     * Mail Domain
     * 
     * @var string
     */
    var $domain;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init()
    {
        require_once 'XML/RPC.php';
        $this->domain = Gadget::getConf('Gadget', 'maildomain');
        $this->client = &new XML_RPC_Client(
            Gadget::getConf('vpop-xmlrpc', 'path'),
            Gadget::getConf('vpop-xmlrpc', 'host'),
            Gadget::getConf('vpop-xmlrpc', 'port')
        );
        $this->client->setCredentials(
            Gadget::getConf('vpop-xmlrpc', 'user'),
            Gadget::getConf('vpop-xmlrpc', 'pass')
        );
    }
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     */
    function execute($in_params)
    {
        list($method, $params) = $in_params;
        
        if (!count($params)) {
            $this->fault = 'Empty params';
            return false;
        }

        if (empty($params['domain'])) {
            $params['domain'] = $this->domain;
        }

        $this->fault   = '';
        $this->method  = strToLower($method);
        $this->account = $params['user'] .'@'. $params['domain'];
        
        // sane parameters?
        switch ($this->method)
        {
            case 'adduser':
                if (!isset($params['user'], $params['password'])) {
                    $this->fault = 'adduser() expects user and password parameters';
                    return false;
                }
                if (!isset($params['fullname'])) {
                    $params['fullname'] = $params['user'];
                }
            break;
            
            case 'deluser':
                if (!isset($params['user'])) {
                    $this->fault = 'deluser() expects user as parameter';
                    return false;
                }
            break;
            
            case 'setquota':
                if (!isset($params['user'], $params['quota'])) {
                    $this->fault = 'setquota() expects user and quota as parameter';
                    return false;
                }
            break;
            
            case 'passwd':
                if (!isset($params['user'], $params['password'])) {
                    $this->fault = 'passwd() expects user and password as parameter';
                    return false;
                }
            break;
            
            case 'auth':
            case 'checkauth':
                if (!isset($params['user'], $params['password'])) {
                    $this->fault = 'auth() extpects user and password as parameter';
                    return false;
                }
            break;
            
            default:
                $this->fault = "Unkown method: {$this->method}";
                return false;
        }
        
        if (PEAR::isError($e = $this->rpcScalar($this->method, $params))) {
            $this->fault = $e->getMessage();
            return false;
        }
        return true;
    }
    
    /**
     * Change Password
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     */
    function passwd($user, $password)
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        return $this->rpcScalar('passwd', $params);
    }
    
    /**
     * Auth
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     */
    function auth($user, $password)
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        return $this->rpcScalar('auth', $params);
    }
    
    /**
     * Check Password
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     */
    function checkAuth($user, $password)
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        return $this->prcScalar('checkauth', $params);
    }
    
    /**
     * Add User
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     * @param   string  $fullname
     */
    function addUser($user, $password, $fullname = '')
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        
        if ($fullname) {
            $params['fullname'] = $fullname;
        } else {
            $params['fullname'] = $params['user'].'@'.$params['domain'];
        }
        
        return $this->rpcScalar('adduser', $params);
    }
    
    /**
     * Delete User
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     */
    function delUser($user)
    {
        $this->userDomain($params, $user);
        return $this->rpcScalar('deluser', $params);
    }
    
    /**
     * Set Quota
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $quota
     */
    function setQuota($user, $quota)
    {
        $params = array('quota' => $quota);
        $this->userDomain($params, $user);
        return $this->rpcScalar('setquota', $params);
    }
    
    /**
     * Add Domain
     * 
     * @access  public
     * @return  bool@PEAR_Error
     * @param   string  $domain
     * @param   string  $password
     */
    function addDomain($domain, $password)
    {
        return $this->rpcScalar('adddomain', 
            array('domain' => $domain, 'password' => $password));
    }
    
    /**
     * Delete Domain
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $domain
     */
    function delDomain($domain)
    {
        return $this->rpcScalar('deldomain', array('domain' => $domain));
    }
    
    /**
     * List Users
     * 
     * @access  public
     * @return  mixed
     */
    function listUsers($domain = null, $offset = 0, $limit = 0, $sort = true)
    {
        isset($domain) or $domain = $this->domain;
        return $this->rpcStruct('listusers', 
            array(
                'domain'    => $domain,
                'limit'     => $limit,
                'offset'    => $offset,
                'sort'      => $sort,
            )
        );
    }
    
    /**
     * List Domains
     * 
     * @access  public
     * @return  mixed
     */
    function listDomains()
    {
        return $this->rpcStruct('listdomains');
    }
    
    /**
     * List Domain
     * 
     * @access  public
     * @return  mixed
     */
    function listDomain($domain = null)
    {
        isset($domain) or $domain = $this->domain;
        return $this->rpcStruct('listdomain', array('domain' => $domain));
    }
    
    /**
     * List User
     * 
     * @access  public
     * @return  mixed
     */
    function listUser($user)
    {
        $this->userDomain($params, $user);
        return $this->rpcStruct('listuser', $params);
    }
    
    /**
     * Mod User
     * 
     * @access  public
     * @return  mixed
     */
    function modUser($user, $fullname = null, $flags = null)
    {
        $this->userDomain($params, $user);
        isset($fullname) and $params['fullname'] = $fullname;
        isset($flags) and $params['flags'] = $flags;
        return $this->rpcScalar('moduser', $params);
    }
    
    
    /**
     * Batch Processing
     * 
     * @access  public
     * @return  mixed
     */
    function batch($action, $params)
    {
        $array = array();
        foreach ((array) $params as $param) {
            $array[] = array_shift($this->_buildStruct($param));
        }
        
        $struct['method'] = &new XML_RPC_Value($action, 'string');
        $struct['values'] = &new XML_RPC_Value($array, 'array');
        
        $value = array(new XML_RPC_Value($struct, 'struct'));
        $result = &$this->_send('batch', $value);
        
        if (PEAR::isError($result)) {
            return $result;
        }
        return $this->flatten($result->xv->me['struct']);
    }
    
    /**
     * User Domain
     * 
     * @access  protected
     * @return  void
     * @param   array   &$params
     * @param   string  $user
     */
    function userDomain(&$params, $user)
    {
        if (preg_match('/(.+)@(.+)/', $user, $matches)) {
            $params['user']     = $matches[1];
            $params['domain']   = $matches[2];
        } else {
            $params['user']     = $user;
            $params['domain']   = $this->domain;
        }
    }
    
    /**
     * RPC with scalar return value
     * 
     * @access  protected
     * @return  bool|PEAR_Error
     * @param   string  $action
     * @param   array   $params
     */
    function rpcScalar($action, $params = array())
    {
        $result = &$this->_rpc($action, $params);
        if (PEAR::isError($result)) {
            return $result;
        }
        return $result->xv->scalarVal();
    }
    
    /**
     * RPC with struct return value
     * 
     * @access  protected
     * @return  array|PEAR_Error
     * @param   string  $action
     * @param   string  $params
     */
    function rpcStruct($action, $params = array())
    {
        $result = &$this->_rpc($action, $params);
        if (PEAR::isError($result)) {
            return $result;
        }
        return $this->flatten($result->xv->me['struct']);
    }
    
    /**
     * RPC with array return value
     * 
     * @access  protected
     * @return  array|PEAR_Error
     * @param   string  $action
     * @param   string  $params
     */
    function rpcArray($action, $params = array())
    {
        $result = &$this->_rpc($action, $params);
        if (PEAR::isError($result)) {
            return $result;
        }
        return $this->flatten($result->xv->me['array']);
    }
    
    /**
     * @access  private
     */
    function &_rpc($action, $params)
    {
        $result = &$this->_send($action, $this->_buildStruct($params));
        return $result;
    }
    
    /**
     * @access  private
     */
    function &_send($action, &$value)
    {
        $message = &new XML_RPC_Message("vpop.$action", $value);
        $message->setSendEncoding(GADGET_CHARSET);
        $result  = &$this->client->send($message);
        if ($result->faultCode()) {
            return PEAR::raiseError($result->faultString(), $result->faultCode());
        }
        return $result;
    }
    
    /**
     * @access  private
     */
    function &_buildStruct($params)
    {
        $value = null;
        if (count($params)) {
            $values = array();
            foreach ((array) $params as $name => $value) {
                $values[$name] = &new XML_RPC_Value($value, $this->_type($value));
            }
            $value = array(new XML_RPC_Value($values, 'struct'));
        }
        return $value;
    }
    
    /**
     * @access  private
     */
    function _type($var)
    {
        switch ($type = gettype($var))
        {
            case 'array':
                if (array_key_exists(0, $var)) {
                    return 'array';
                }
            case 'object':  return 'struct';
            case 'string':  return 'string';
            case 'integer': return 'int';
            case 'double':  return 'double';
            case 'boolean': return 'boolean';
            default:        return '';
        }
    }
    
    /**
     * Flatten Structure
     * 
     * @access  protected
     * @return  array
     * @param   array   $struct
     */
    function flatten(&$struct)
    {
        $result = array();
        foreach ((array) $struct as $key => $val) {
            switch ($type = $val->kindOf())
            {
                case 'array';
                case 'struct':
                    $result[$key] = $this->flatten($val->me[$type]);
                break;
                
                case 'undef':
                    $result[$key] = $val->getVal();
                break;
                
                default:
                    $result[$key] = $val->scalarVal();
            }
        }
        return $result;
    }
    
    /**
     * On Success
     * 
     * @access  protected
     * @return  bool true
     */
    function onSuccess()
    {
        return parent::onSuccess($this->method .'('. $this->account .')');
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure()
    {
        return parent::onFailure($this->method .'('. $this->account .') '. $this->fault);
    }
    
    /**
     * Safe Name
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $name
     */
    function safeName($name)
    {
        return iconv(GADGET_CHARSET, 'ASCII//IGNORE', 
            preg_replace('/[^\w._-]+/', '_', $name));
    }
}

?>
