<?php

/**
 * Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.4 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Render.php,v 1.4 2014-01-14 20:07:40 mike Exp $
 */

/**
 * Requires Gadget_Service
 */
require_once 'Gadget/Service.php';

/** 
 * Gadget_Service_Render
 *
 * Gadget's default Render Service using Gadget_Template.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Render extends Gadget_Service
{
    var $service = SERVICE_RENDER;
    
    var $GTE;
    var $buffer;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init()
    {
        require_once 'Gadget/Template.php';
        $this->GTE = &new Gadget_Template(Gadget::getConf('Gadget_Template'));
    }
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  mixed
     * @param   object Gadget_Widget $widget
     * @param   bool    $get
     */
    function execute($widget, $get = false)
    {
        if (!$get && !$widget->show) {
            return false;
        }
        
        if ($widget->show && !$widget->template) {
            PEAR::raiseError(get_class($widget) .' has no template defined');
            return false;
        }
        
        $this->GTE->compile($widget->template);
        
        // QuickForm
        // Replace $widget->form with the rendered form 
        if ($widget->qform) {
            $rend = &Gadget::persistentObject('HTML_QuickForm_Renderer_Gadget');
    
            $widget->qform->accept($rend);
            $widget->form = &$rend->form;
        }
        
        if ($get) {
            return $this->buffer = $this->GTE->renderWidget($widget);
        } else {
            return $this->GTE->displayWidget($widget);
        }
    }
    
    /**
     * On Success
     * 
     * @access  protected
     * @return  mixed
     */
    function onSuccess()
    {
        $return = parent::onSuccess();
        if ($this->buffer) {
            $return = $this->buffer;
            $this->buffer = null;
        }
        return $return;
    }
    
}

?>
