<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.4 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Template.php,v 1.4 2007-03-31 06:01:54 mike Exp $
 */

/**
 * Requires Gadget_Object
 */
require_once 'Gadget/Object.php';

/**
 * Requires Gadget_Template_Compiler
 */
require_once 'Gadget/Template/Compiler.php';

/**
 * Gadget_Template
 *
 * Gadget's Template Engine.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @since       1.0
 */
class Gadget_Template extends Gadget_Object
{
    /**
     * Template Dir 
     *
     * @var string
     */
    var $templateDir = 'templates/';
    
    /**
     * Compiled Dir 
     *
     * @var string
     */
    var $compiledDir = 'templates/_c/';
    
    /**
     * Compiled Template 
     *
     * @var string
     */
    var $compiledTemplate;
    
    /**
     * Prepared Template 
     *
     * @var string
     */
    var $preparedTemplate;
    
    /**
     * Compiler 
     *
     * @var object Gadget_Template_Compiler
     */
    var $Compiler;
    
    /**
     * Force Compile 
     *
     * @var bool
     */
    var $forceCompile = false;
    
    /**
     * Need Compile
     * 
     * @var bool
     */
    var $needCompile = true;
    
    /**
     * Compile
     *
     * @access   public
     * @return   bool
     * @param    string    $template
     */
    function compile($template)
    {
        list($prepared, $compiled) = $this->prepareTemplate($template);
        
        if (!$this->needCompile) {
            return true;
        }
        
        if (    !$this->forceCompile && is_file($compiled) && 
                filemtime($prepared) <= filemtime($compiled)) {
            return true;
        }
        
        if (!$this->Compiler) {
            $this->Compiler = &Gadget_Template_Compiler::singleton();
        }

        return $this->Compiler->compile($prepared, $compiled);
    }
    
    /**
     * Prepare Template
     * 
     * @access  protected
     * @return  array
     * @param   string  $template
     */
    function prepareTemplate($template)
    {
        $this->preparedTemplate = $this->templateDir .'/'. $template;
        if ($this->needCompile) {
            $this->compiledTemplate = $this->compiledDir .'/'. $template .'.php';
        } else {
            $this->compiledTemplate = $this->preparedTemplate;
        }
        
        return array($this->preparedTemplate, $this->compiledTemplate);
    }
    
    /**
     * Display
     * 
     * @access  public
     * @return  bool
     * @param   array   $data
     */
    function display($data = array())
    {
        extract($data);
        return include $this->compiledTemplate;
    }
    
    /**
     * Render
     * 
     * @access  public
     * @return  string
     * @param   array   $data
     */
    function render($data)
    {
        ob_start();
        $this->display($data);
        return ob_get_clean();
    }
    
    /**
     * Display Widget
     * 
     * @access  public
     * @return  bool
     * @param   object Widget   &$widget
     */
    function displayWidget(&$widget)
    {
        foreach ((array) array_keys((array) get_object_vars($widget)) as $p) {
            $$p = &$widget->$p;
        }
        return include $this->compiledTemplate;
    }
    
    /**
     * Render Widget
     * 
     * @access  public
     * @return  string
     * @param   object Widget   &$widget
     */
    function renderWidget(&$widget)
    {
        ob_start();
        $this->displayWidget($widget);
        return ob_get_clean();
    }
}
?>
