<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Node.php,v 1.3 2007-03-30 11:40:08 mike Exp $
 */

/**
 * Requires Gadget_Tree_Node
 */
require_once 'Gadget/Tree/Node.php';

/**
 * Gadget_Tree_XML_Node
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_XML_Node extends Gadget_Tree_Node
{
    var $self = __CLASS__;
    var $namespace = '';
    var $isTextNode = false;
    
    /**
     * Constructor
     * 
     * @access  public
     */
    function Gadget_Tree_XML_Node($name)
    {
        if (empty($name)) {
            $this->isTextNode = true;
        } elseif (strstr($name, ':')) {
            list($this->namespace, $new_name) = explode(':', $name, 2);
            $name = $new_name;
        }
        parent::Gadget_Tree_Node($name);
    }
    
    /**
     * Set Meta
     * 
     * @access public 
     * @return mixed 
     */
    function setMeta($meta)
    {
        if ($this->isTextNode) {
            return Gadget_Tree::raiseError(
                'Cannot set meta information of a text node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        return parent::setMeta($meta);
    }

    /**
     * Add Meta
     * 
     * @access public 
     * @return mixed 
     */
    function addMeta($meta, $value = null)
    {
        if ($this->isTextNode) {
            return Gadget_Tree::raiseError(
                'Cannot add meta information to a text node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        return parent::addMeta($meta, $value);
    }

    /**
     * Attribute String
     * 
     * @access  public
     * @return  string
     */
    function attributeString()
    {
        if ($this->isTextNode) {
            return Gadget_Tree::raiseError(
                'A text node has no attributes',
                GADGET_TREE_INVALID_OP
            );
        }
        $attributes = '';
        foreach ($this->meta as $attr => $val){
            $attributes .= ' '. $attr .'="'. addCslashes($val, '"') .'"';
        }
        return $attributes;
    }
}
?>