<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: PageHeader.php,v 1.3 2007-03-30 11:40:09 mike Exp $
 */

/**
 * Requires Gadget_Widget
 */
require_once 'Gadget/Widget.php';

/**
 * PageHeader
 *
 * The page header widget.
 * Provides routines for adding style sheets, javascripts, links etc.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @since       1.0
 */
class PageHeader extends Gadget_Widget
{
    var $addStyle;
    var $addScript;
    var $styleSheets = array();
    var $javaScripts = array();
    
    var $otherLinks = array();
    
    var $allowRobots = true;
    
    var $title = '';
    var $language;
    var $charset;
    
    var $template = 'PageHeader.html';
    
    /**
     * On Before Construct
     * 
     * @access  protected
     * @return  void
     */
    function onBeforeConstruct()
    {
        parent::onBeforeConstruct();
        $this->language = Gadget::getPref('language');
        $this->charset  = Gadget::getPref('charset');
        if ($_SERVER['PHP_SELF'] !== '/index.php') {
            $this->addLink('top', '/', 'MonA-Net');
        }
    }
    
    function main() {}
    
    /**
     * Get Script
     * 
     * @access  public
     * @return  string
     */
    function getScript()
    {
        return  "\n\t<script type=\"text/javascript\"> <!--".
                "\n\t\t". $this->addScript ."\n// -->\n\t</script>\n";
    }
    
    /**
     * Add Script
     * 
     * @access  public
     * @return  void
     * @param   string
     */
    function addScript($script)
    {
        $this->addScript .= $script . "\n";
    }
    
    /**
     * Add Style
     * 
     * @access  public
     * @return  void
     * @param   string
     */
    function addStyle($style)
    {
        $this->addStyle .= $style . "\n";
    }
    
    /**
     * addStyleSheet
     *
     * @access  public
     * @return  void
     * @param   string
     * @param   string
     * @param   string
     */
    function addStyleSheet($sheet, $ext = '.css', $base = '/_css/')
    {
        if(!in_array($styleSheet = $base . $sheet . $ext, $this->styleSheets))
        $this->styleSheets[] = $styleSheet;
    }
    
    /**
     * addJavaScript
     *
     * @access  public
     * @return  void
     * @param   string
     * @param   string
     * @param   string
     */
    function addJavaScript($script, $ext = '.js', $base = '/_js/')
    {
        if(!in_array($javaScript = $base . $script . $ext, $this->javaScripts))
        $this->javaScripts[] = $javaScript;
    }
    
    /**
     * addLink
     *
     * @access  public
     * @return  void
     * @param   string
     * @param   string
     * @param   string
     */
    function addLink($rel, $href, $title = null)
    {
        $this->otherLinks[$rel] = (object) array('href' => $href, 'title' => $title);
    }
}

?>
